@@ -1,12 +1,29 @@
-#!/usr/bin/python
-
-# Created by: Anders Israelsson @ Uppsala University, 2023
+#!/usr/bin/python3
+' '
+# Created by: Anders Israelsson, 2023
+# Email: andis854@outlook.com
+# https://github.com/andis854/matrix_rdn_det
 
 import numpy
 
 # det_int calculates the determinant of matrices with interger entries
 # and reduces the number of calculations whenever an entry is 0.
 def det_int(matrix):
+    '''Calculate the determinant of a square matrix with integer entries. 
+    
+    The function expands rows and columns recursively and reduces calculation by 
+    finding the rows/columns with the largest number of zeroes.
+    
+    Parameters
+    ----------
+    matrix : numpy.array
+        Numpy array representing a square matrix, which determinant is to be evaluated.
+    
+    Returns
+    -------
+    determinant : int
+        The determinant of the input matrix.
+         '''
     dimension = numpy.size(matrix, axis=0)
     if dimension != numpy.size(matrix, axis=1):
         raise TypeError('Not a square matrix!')
@@ -51,6 +68,7 @@ def det_int(matrix):
 # denominator*div_rest[0]+div_rest[1] and abs(div_rest[1]) is the
 # smallest possible value.
 def divmod_mod(numerator, denominator): 
+    '\b'
     if ((not type(numerator) is int) or (not type(denominator) is int)) and ((not type(numerator) is numpy.int64) or (not type(denominator) is numpy.int64)):
         raise TypeError('Input must be of type int or numpy.int64')
     div_rest = numpy.array(numpy.divmod(numerator, denominator))
@@ -64,6 +82,7 @@ def divmod_mod(numerator, denominator):
 
 # Calculate the greatest common divisor of a numpy array of integers.
 def gcd(array):
+    '\b'
     if not type(array) is numpy.ndarray:
         raise TypeError('Input must be of type numpy.ndarray')
     if numpy.size(array) > 2:
@@ -80,32 +99,30 @@ def numpy2latex(matrix,_print=False):
     Parameters
     ----------
     matrix : numpy.array
-        Array that is written LaTeX friendly. 
+        Array that will be written LaTeX friendly. 
     det_value : bool, optional
-        If true, print the output in terminal. 
+        If true, print the output in terminal. Otherwise return as a string.
     
     Returns
     -------
     output : str
-        A numpy array with shape [dimension,dimension].
+        If _print is set to 'False' the function returns a string that is LaTeX friendly when printed. 
     
-
     Notes
     -----
-    The output can be used together with LaTeX environment, e.g.
-    \begin{pmatrix}
+    The output can be used together with some LaTeX environments, e.g.
+    \\begin{pmatrix}
         [output]
-    \end{pmatrix}
+    \\end{pmatrix}
     or
-    \begin{array}
+    \\begin{array}
         [output]
-    \end{array}
-    
+    \\end{array}
     
     Examples
     --------
     >>> numpy2latex(numpy.array([[4,-3],[5,2],[-8,5]]))
-    '4 & -3 \\\\\n5 & 2 \\\\\n-8 & 5'
+    4 & -3 \\\\\\\\\\n5 & 2 \\\\\\\\\\n-8 & 5'
     
     >>> numpy2latex(numpy.array([[4,-3],[5,2],[-8,5]]),_print=True)
     4 & -3 \\
@@ -138,39 +155,33 @@ def numpy2latex(matrix,_print=False):
         return output
 
 
-
-
-
-
-
-
-
-
-
-
 # matrix_gen outputs a random matrix with requested value of the determinant.
 def matrix_gen(dimension = 2, det_value = 1, lower_bound = -9, upper_bound = 10, rdn_prm = 0, attempts=200):
 
-# The function works like this: a matrix of specified dimension is requested. First, the rows 2:dimension are randomized. Then the possible values first row is calculated by solving the diophantine equation
+# In this function a matrix of specified dimension is requested. First, the rows [2:dimension] are randomized. 
+# Then the possible values first row is calculated by solving the diophantine equation
 
-    # a_1*c_1+a_2*c_2+...+a_dimension*c_dimension = det_value
+    # a_1*c_1+a_2*c_2+...+a_dimension*c_dimension = det_value.
 
 # where c_1,...,c_dimension are the cofactors of the first row of the matrix. 
-# This is done by substuting the variables with new variables until the 
+# The diophantine equation is solved by substuting the variables with new variables until the 
 # diophantine equation has only two unknown variabes. The program always try to
 # reduce the term with largest cofactor (in absolute value).
 
-  # Ex: assume that the diophantine equation is calculated as
+  # Example: assume that the diophantine equation is calculated as
 
     # 5a_1 + 7a_2 + 8a_3 = 1
 
   # Then make the following substitutions: (start with reducing 8 in the third term)
 
-    # a_2 = -a_3 + b_1 => 5a_1 +7b_1+a_3 = 1
-    # a_1 = -b_1 + b_2 => 5b_2 +2b_1 + a_3 = 1
-    # b_1 = -2b_2 + b_3 => b_2 + 2b_3 + a_3 = 1
-    # b_2 = -2b_3 + b_4 => b_4 + a_3 = 1
-    # => b_4 + a_3 = 1
+    # a_2 = -a_3 + b_1 => 5a_1 +7b_1+a_3 = 1,
+    # a_1 = -b_1 + b_2 => 5b_2 +2b_1 + a_3 = 1,
+    # b_1 = -2b_2 + b_3 => b_2 + 2b_3 + a_3 = 1,
+    # b_2 = -2b_3 + b_4 => b_4 + a_3 = 1,
+    # => b_4 + a_3 = 1.
+  
+  # That is, we have a modified and lot simpler diophantine equation. 
+  # Note that this shows that the original diophantine equation is solvable.
   # Moving everything to the left hand side we obtain the augmented matrix
   # [[0,1,1,1,0,0,0,0],
   #  [1,0,0,1,-1,0,0,0],
@@ -221,7 +232,7 @@ def matrix_gen(dimension = 2, det_value = 1, lower_bound = -9, upper_bound = 10,
 
     Notes
     -----
-    This can be used for e.g. linear algebra teachers that want to create system of equations exercises 
+    This can be used for e.g. teachers in linear algebra who want to create system of equations exercises 
     that are easy to solve by hand and possibly where fractions are avoided 
     (if det_value is chosen to be \u00B11).
     
@@ -301,10 +312,6 @@ def matrix_gen(dimension = 2, det_value = 1, lower_bound = -9, upper_bound = 10,
            [ 1,  6,  6,  7,  7,  2]])"""
 
 
-
-
-
-    
     if not type(dimension) is int:
         raise TypeError('Only integers are allowed for dimension')
     if dimension <= 0: # Take care of the special cases
@@ -330,20 +337,12 @@ def matrix_gen(dimension = 2, det_value = 1, lower_bound = -9, upper_bound = 10,
     elif dimension <= 3 and lower_bound >= upper_bound-2 and (lower_bound >= 2 or upper_bound <=-1):
         raise ValueError('the values of lower_bound and upper_bound are too narrowly chosen!')
 
-
-
-
-      
     # If rows are swapped in the end, the sign of the determinant changes.
     rdn_row = numpy.random.randint(0,dimension) 
     if rdn_row != 0:
         det_value = -det_value
 
 
-
-
-
-
     # If parameters is randomized below, choose how many attempts before restarting the function.
     sol_attempts = attempts
     while attempts <= sol_attempts:
@@ -440,11 +439,11 @@ def matrix_gen(dimension = 2, det_value = 1, lower_bound = -9, upper_bound = 10,
                     coeff_geq_2 = numpy.size(cofactors[numpy.abs(cofactors) > 1])
                     row_counter += 1
 
-        
+                
                 final_equation = numpy.concatenate((cofactors, numpy.array([-det_value])))
                 final_equation = numpy.array(final_equation/gcd(final_equation),int)
 
-                if numpy.count_nonzero(final_equation) >= 3 or det_value % final_equation[order[0]] == 0: # check if solution exists. This can happen if there are 
+                if numpy.count_nonzero(final_equation) >= 3 or final_equation[-1] % final_equation[order[0]] == 0: # check if solution exists. This can happen if there are 
                     # either >= 2 cofactors left (since only one cofactors can have absolute 
                     # value > 1, gcd of the coefficients has to be 1) or if there is 1 
                     # coefficients which divides the right hand side.
@@ -466,22 +465,17 @@ def matrix_gen(dimension = 2, det_value = 1, lower_bound = -9, upper_bound = 10,
                 return matrix
         
 
-
-
         while row_counter < _dim - 1: # Makes sure every unknown variable is written as function 
             # of a parameter, in case the system of equation contains too few rows.
             sys_of_eq[row_counter, row_counter] = 1
             sys_of_eq[row_counter, row_counter + _dim] = -1
             row_counter += 1
 
-
-
-          
         # Adding final equation the system of equations
         row_counter += 1
         
-        sys_of_eq[row_counter - 1, row_counter + _dim - 1] = -det_value
-        sys_of_eq[row_counter - 1, coeff_enumeration] = cofactors
+        sys_of_eq[row_counter - 1, row_counter + _dim - 1] = final_equation[-1]
+        sys_of_eq[row_counter - 1, coeff_enumeration] = final_equation[:-1]
         
 
 
@@ -612,8 +606,9 @@ def matrix_gen(dimension = 2, det_value = 1, lower_bound = -9, upper_bound = 10,
             else:
                 randomised_parameters = numpy.random.choice(numpy.delete(numpy.arange(0,_dim-1),control_rows[0:-1]),rdn_prm,replace=False) # Chooses variables from the set of 
                 # unknowns depending on several parameters.
-                b[randomised_parameters] = numpy.random.randint(parameters_bounds[randomised_parameters,0],parameters_bounds[randomised_parameters,1]+1)
-                non_randomised_parameters = numpy.delete(numpy.arange(0,_dim-1),randomised_parameters)
+                for randomised_parameter_counter in randomised_parameters:
+                    b[randomised_parameter_counter] = numpy.random.randint(parameters_bounds[randomised_parameter_counter,0],parameters_bounds[randomised_parameter_counter,1]+1)
+                    non_randomised_parameters = numpy.delete(numpy.arange(0,_dim-1),randomised_parameter_counter)
 
             
     
@@ -650,11 +645,7 @@ def matrix_gen(dimension = 2, det_value = 1, lower_bound = -9, upper_bound = 10,
         
                 b[non_randomised_parameters[counter]] += 1
             
-        
-        
-        
-        
-            
+      
         # Part 6 - Choose one solution and concatenate it with the rest of the randomized
         # matrix rows.
         
@@ -669,8 +660,6 @@ def matrix_gen(dimension = 2, det_value = 1, lower_bound = -9, upper_bound = 10,
 
             matrix = numpy.concatenate((solution, matrix_red))
         
-        
-        
     
     matrix[[0,rdn_row],:] = matrix[[rdn_row,0],:]
     
@@ -681,12 +670,12 @@ if __name__ == '__main__':
     import sys
     
     _epilog='''examples:
-  matrix_rdn_det 
+  matrix_rdn_det.py 
                         Outputs a matrix of dimension 2 with determinant 1 and entries between -9 and 9.
                         Example output:
                           9 & -1 \\\\
                           1 & 0
-  matrix_rdn_det 5
+  matrix_rdn_det.py 5
                         Outputs a matrix of dimension 3 with determinant 1 and entries between -9 and 9.
                         Example output:
                           -2 & 3 & 5 & 7 & 7 \\\\
@@ -694,7 +683,7 @@ if __name__ == '__main__':
                           4 & -4 & -5 & 5 & 4 \\\\
                           -9 & -9 & 1 & 5 & 0 \\\\
                           -9 & -7 & 4 & 3 & -3
-  matrix_rdn_det 5 -4
+  matrix_rdn_det.py 5 -4
                         Outputs a matrix of dimension 5 with determinant -4 and entries between -9 and 9.
                         Example output:
                           6 & 3 & 6 & 4 & -8 \\\\
@@ -702,7 +691,7 @@ if __name__ == '__main__':
                           9 & -8 & -7 & -5 & 5 \\\\
                           0 & -1 & -2 & 3 & -8 \\\\
                           -2 & -6 & -1 & -1 & -7
-  matrix_rdn_det 5 6 -3 14
+  matrix_rdn_det.py 5 6 -3 14
                         Outputs a matrix of dimension 5 with determinant 6 and entries between -3 and 13.
                         Example output:
                           11 & 1 & 4 & 3 & -1 \\\\
@@ -710,7 +699,7 @@ if __name__ == '__main__':
                           0 & 4 & 4 & 8 & 10 \\\\
                           -2 & 6 & 3 & 11 & 8 \\\\
                           -3 & 3 & 11 & 1 & 11
-  matrix_rdn_det 7 6 -3 14 2 300
+  matrix_rdn_det.py 7 6 -3 14 2 300
                         Outputs a matrix of dimension 7 with determinant 6 and entries between -3 and 13. The number
                         of randomization of 2 parameters can make the calculations faster. The parameters are 
                         randomized 300 times before the called function restarts. It is recommended to set randomized 
@@ -725,9 +714,29 @@ if __name__ == '__main__':
                           -1 & 7 & 13 & 5 & 6 & -3 & 12'''
     parser = argparse.ArgumentParser(description='''                        Randomize a matrix with the determinant value as a parameter. The output is LaTeX compatible.
     
-                        This can be used for e.g. linear algebra teachers that want to create system of equations exercises 
-                        that are easy to solve by hand and possibly where fractions are avoided 
-                        (if det_value is chosen to be \u00B11).''', formatter_class=argparse.RawTextHelpFormatter, epilog=_epilog)
+                        This can be used for e.g. teachers in linear algebra who want to create system of equations 
+                        exercises that are easy to solve by hand and possibly where fractions are avoided 
+                        (if det_value is chosen to be \u00B11).
+                        
+                        When the dimension is 2 or 3 the difference between lower_bound and upper_bound has to be at
+                        least 3 (except if lower_bound <= -1 and upper_bound >=1). This is to ensure that the problem
+                        is solvable. Otherwise the difference between lower_bound and upper_bound has to be at least 2.
+
+                        If the dimension is set to 7 or higher, it is recommended to set a few random parameters to
+                        speed up the calculcations. However, the randomness of the entries will decrease. The number
+                        of attemps is only used if there are randomized parameters set.
+    
+                        The standard output can be used together with some LaTeX environments, e.g.
+                        \\begin{pmatrix}
+                            [output]
+                        \\end{pmatrix}
+                        or
+                        \\begin{array}
+                            [output]
+                        \\end{array}
+                        
+                        https://github.com/andis854/matrix_rdn_det
+                        Report bugs to andis854@outlook.com''', formatter_class=argparse.RawTextHelpFormatter, epilog=_epilog)
 
     parser.add_argument('parameters', nargs='*', help='''Set the parameters [dimension, det_value, lower_bound, upper_bound, rdn_prm, attempts]. 
           dimension
